package org.oscy.tapestry.breadcrumbs.session.impl;

import java.util.ArrayList;
import java.util.List;

import org.oscy.tapestry.breadcrumbs.other.BreadCrumbElement;
import org.oscy.tapestry.breadcrumbs.session.BreadCrumbsList;

/**
 * Implementation class for {@link BreadCrumbsList}
 * <p/>
 * All operations on the breadcrumbs list are protected with a mutex in order
 * to avoid concurrent modifications.
 */
public class BreadCrumbsListImpl implements BreadCrumbsList
{
	private static final long serialVersionUID = -8457209111821626779L;
	private final ArrayList<BreadCrumbElement> items;
	private final Object mutex;
	private final int size;
	private final boolean discardDuplicates;

	/**
	 * Default constructor for that service.All the parameters are injected through 
	 * tapestry-ioc.
	 * 
	 * @param size Maximum breadcrumbs to save on the list
	 * @param discardDuplicates Whether to discard duplicated breadcrumbs
	 * 
	 * @see org.oscy.tapestry.breadcrumbs.other.BreadCrumbsConstants#BREADCRUMBS_MAX_CRUMBS_TO_SAVE
	 * @see org.oscy.tapestry.breadcrumbs.other.BreadCrumbsConstants#BREADCRUMBS_DISCARD_DUPLICATES
	 */

	public BreadCrumbsListImpl(int size, boolean discardDuplicates) {
		this.items = new ArrayList<BreadCrumbElement>(size);
		this.mutex = new Object();
		this.size = size;
		this.discardDuplicates = discardDuplicates;
	}

	
	/**
	 * Clear the whole breadcrumbs list
	 * <p/>
	 * This function is invoked from {@link org.oscy.tapestry.breadcrumbs.internal.services.impl.BreadCrumbsServiceImpl}
	 * when it finds a contribution that specifies {@link  org.oscy.tapestry.breadcrumbs.other.BreadCrumbAttribute#RESET} attribute or
	 * a {@link org.oscy.tapestry.breadcrumbs.annotation.BreadCrumb} that specifies {@link org.oscy.tapestry.breadcrumbs.annotation.BreadCrumb#reset()} 
	 */
	public void clear() {
		synchronized (mutex) {
			items.clear();
		}
	}
	
	/**
	 * Function responsible for adding the specified breadCrumb on the list
	 * <p/>
	 * If the breadcrumb is already the last element of the list then its ignored.This is 
	 * useful in order to avoid post after redirect patters.
	 * <p/>
	 * If discardDuplicates is true it also clears the list starting from the last breadcrumb
	 * which is the same as the passed parameter
	 * 
	 * @param breadCrumb BreadCrumb to add to the list
	 * 
	 */
	public void add(BreadCrumbElement breadCrumb) {
		synchronized (mutex) {
			final int index = items.lastIndexOf(breadCrumb);
			/*
			 * No need to add the page since is exactly the same as the previous
			 * one
			 */
			if (index != -1 && index == items.size() - 1) {
				return;
			}
			if (discardDuplicates) {
				if (index != -1) {
					items.subList(index + 1, items.size()).clear();
					return;
				}
			} else {
				while (items.size() == size) {
					items.remove(0);
				}
			}
			items.add(breadCrumb);
		}
	}

	public List<BreadCrumbElement> getBreadCrumbs() {
		synchronized (mutex) {
			return new ArrayList<BreadCrumbElement>(items);
		}
	}
}
